#include <stdio.h>
#include <avr/io.h>

#include "SHT15.h"

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

//Init the sensor and read out the humidity and temperature data
char sht15_read(float *temperature_o, float * humidity_o)
{
//	float humidity, temperature;
    int32_t response;
//    int32_t humidity_sqr;
    
	sht15_start(); //Issue transmission start
    sht15_send_byte(CHECK_HUMD); //Now send command code
    while ( (PINC & (1 << SHT_SDA)) ); //Wait for SHT to pull line low
    response = sht15_read_byte(); //Read 8MSB
    response <<= 8;
    response |= sht15_read_byte(); //Read 8LSB
	*humidity_o = -2.0468 + 0.0367*response + (-1.5955E-6)*response*response;

    sht15_start();
    sht15_send_byte(CHECK_TEMP);
    while ( (PINC & (1 << SHT_SDA)) ); //Wait for SHT15 to pull SDA low to signal measurement completion
    response = sht15_read_byte(); //Read 8MSB
    response <<= 8;
    response |= sht15_read_byte(); //Read 8LSB
	*temperature_o = (-39.7) + 0.01 * response;
	
	return 0;
}

void sht15_send_byte(uint8_t sht15_command)
{
    uint8_t i;

    WRITE_sda();

    for(i = 8 ; i > 0 ; i--)
    {
        cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
        delay_ms(1);
        
        if(sht15_command & (1 << (i-1)))
        {
            sbi(PORTC, SHT_SDA);
        }
        else
        {
            cbi(PORTC, SHT_SDA);
        }

        sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
        delay_ms(1);
    }

    //Wait for SHT15 to acknowledge.
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    //0 = input, 1 = output
    READ_sda();
    while ( PINC & (1 << SHT_SDA)); //Wait for SHT to pull line low

    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0; //Falling edge of 9th clock
    delay_ms(1);
    
    //while ( (PINC & (1 << SHT_SDA)) == 0); //Wait for SHT to release line


}

//Specific SHT start command
void sht15_start(void)
{
    WRITE_sda();
    sbi(PORTC, SHT_SDA); //SHT_SDA = 1;
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);

    cbi(PORTC, SHT_SDA); //SHT_SDA = 0;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
    delay_ms(1);
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    sbi(PORTC, SHT_SDA); //SHT_SDA = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
    delay_ms(1);
}

//Read 8 bits from the SHT sensor
uint8_t sht15_read_byte(void)
{
    uint8_t j;
    uint8_t in_byte = 0;

    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    READ_sda();

    for(j = 0 ; j < 8 ; j++)
    {
        cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
        delay_ms(1);
        sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
        delay_ms(1);

        //Read bit off bus
        in_byte = in_byte << 1;
        if ( (PINC & (1 << SHT_SDA)) ) in_byte |= 1; //in_byte.0 = SHT_SDA;
    }

    //Send acknowledge to SHT15
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    WRITE_sda();
    cbi(PORTC, SHT_SDA);
    
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0; //Falling edge of 9th clock
    delay_ms(1);

    return(in_byte);
}

void sht15_init(void)
{
	sbi(DDRC, SHT_SCK);
	sbi(DDRC, SHT_SDA);

//    DDRB = 0b11101111; //PB4 = MISO 
//    DDRC = 0b11111010; //PC0 = DRDY, PC2 = SCP_MISO, PC4 = SDA, PC5 = SCL
//    DDRD = 0b11011110; //PORTD (RX on PD0) PD5 = MISO
}

