/*
https://www.brisbanehotairballooning.com.au/faqs/education/113-pressure-altitude-conversion.html
http://www.hochwarth.com/misc/AviationCalculator.html
http://www.srh.noaa.gov/epz/?n=wxcalc
http://www.engineeringtoolbox.com/air-altitude-pressure-d_462.html
http://www.sparkfun.com/products/8161
*/

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <math.h>
#include "SHT15.h"
#include "SCP1000-D01.h"


#define FOSC F_CPU // get from makefile

#define BAUD 9600
#define MYUBRR (((((FOSC * 10) / (16L * BAUD)) + 5) / 10) - 1)

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

#define STATUS_LED 			3

//#define VERBOSE				1
//#define MINIMAL				0

#define TEMP_OFFSET_LOC		0
#define OUTPUT_TYPE_LOC		1
#define REPORT_BATT_LOC		2
#define REPORT_LIGHT_LOC	3

#define BATT_CHANNEL		6
#define LIGHT_CHANNEL		7

//#define REPORT 				1
//#define NO_REPORT 			0


static int uart_putchar(char c, FILE *stream);
uint8_t uart_getchar(void);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
uint16_t check_adc(uint8_t channel);

//void print_decimal(int32_t number, uint8_t place, uint8_t pad);
void delay_ms(uint16_t x); // general purpose delay

//void config_menu(void);

int32_t scp_temperature;
int32_t scp_temperature_inF;
uint32_t pressure;
uint16_t counter;

uint16_t global_clock;
volatile uint8_t flag;

volatile uint8_t output_type;
volatile int8_t temp_offset;
volatile uint8_t report_light;
volatile uint8_t report_batt;
volatile uint16_t light_level;
volatile uint16_t batt_level;

void uart_init(void)
{
	UBRR0 = MYUBRR;

	// Enable receiver and transmitter
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	// Set frame format: 8data, 2stop bit
	UCSR0C = (1<<USBS0)|(3<<UCSZ00);

	//Required for printf init
  	stdout = &mystdout; 	
}

int main (void)
{
    uint8_t choice;
	counter = 0;
    
    uart_init();
	scp_init();
    sht15_init();
    
    //ioinit (); //Boot up defaults

repeat_it:
    while(1)
    {
    	float humidity, temperature;
    	float temperature2, pressure;
	float altitude = 0; // =(1-(B3/101325)^0.190284)*145366.45
	int light;
	
	light = check_adc(LIGHT_CHANNEL);

	    unsigned char er = scp_read(&temperature2, &pressure); 
	    if(er == -1)	
	    	goto repeat_it;
		sht15_read(&temperature, &humidity);
		
		altitude = (1-pow((pressure / 101325),0.190284))*145366.45;
		

		printf("t1:%3.2f, t2:%3.2f, hum:%3.2f, pres:%3.2f, light:%04d\n", (double)temperature, (double)temperature2, (double)humidity, (double)pressure, light);


		for(volatile int i = 0; i<10000; i++){}
	}
}




//Check a given channel on the ADC
uint16_t check_adc(uint8_t channel)
{
	#define AVERAGE 16
	
	uint16_t temp, result;
	uint8_t i;
	
	ADMUX = (1<<REFS0)|(channel&0x0F); //Select channel
	
	result = 0;
	for(i = 0 ; i < AVERAGE ; i++)
	{
		ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);//|(1<<ADFR);
		
		//Conversion starts - wait for ADSC to clear
		while(ADCSRA & (1<<ADSC));
		
		temp = ADCL;
		temp |= (uint16_t)(ADCH<<8);
		
		result += temp;
	}
	ADCSRA = 0; //Disable ADC

	return(result/AVERAGE);
}


static int uart_putchar(char c, FILE *stream)
{
  if (c == '\n')
    uart_putchar('\r', stream);
  
  //while ( !( UCSRA & (1<<UDRE)) );
  
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
  return 0;
}


