#include "i2c.h"

void i2c_init()
{
    set(I2CM_DDR_SCL, I2CM_SCL);
    set(I2CM_DDR_SDA, I2CM_SDA);    
}

void i2c_start()
{
    // initialize bits for safety
    clr(I2CM_PORT_SCL, I2CM_SCL);
    clr(I2CM_PORT_SDA, I2CM_SDA);    
    
    // start condition: data changes to low while clock is high
    DLY
    SDA_1
    DLY
    SCL_1
    DLY
    SDA_0
    DLY
    SCL_0
    DLY
}

void i2c_stop(void)
{
	// stop condition: data changes to high while clock is high
    DLY
    SDA_0
    DLY
    SCL_1
    DLY
    SDA_1
    DLY
}

unsigned char i2c_send(unsigned char data)
{
    register unsigned char i;
    int returnas;

    // for all bits of data byte
    for(i=0; i<=7; i++)
    {
    	// set data line according to bit
		if ( data & 0x80) 
	    	{ SDA_1 }
		else
	    	{ SDA_0 }
	    DLY
	    // send a clock pulse
		SCL_1
		DLY
		SCL_0
		DLY
		
		// get next bit
		data = data << 1;
    }
    // trigger acknowledge
    RELEASE_SDA
    SCL_1
    DLY
    // test if acknowledge is received
    if(check(I2CM_PIN_SDA, I2CM_SDA) == 0 ){returnas = 0;} else {returnas = 1;}
    DLY
	SCL_0
	DLY	
return returnas;
}


unsigned char i2c_receive(unsigned char end)
{
	unsigned char i,data = 0;
    // just to be sure
//	DLY
    RELEASE_SDA
    
    // for all bits
	for(i=0; i<=7; i++)
	{
	    // set clock pulse
		DLY
		SCL_1
//		DLY
		// get next data bit
		data <<= 1;
		if(check(I2CM_PIN_SDA, I2CM_SDA) == 1<<I2CM_SDA)
			data++;
		// reset clock pulse
		DLY
		SCL_0
		DLY
	}
    // acknowledge if more is expected
	if(end==I2CM_CONTINUE)
		SDA_0
	else
		SDA_1
		
	// clock pulse for acknowledge
//	DLY
	SCL_1
	DLY
	SCL_0
	DLY

	return data;
}
